@@ -6,7 +6,7 @@ A simple example that exposes the desktop directory as a resource.
 
 from pathlib import Path
 
-from fastmcp.server import FastMCP
+from fastmcp import FastMCP
 
 # Create server
 mcp = FastMCP("Demo")
@@ -19,6 +19,13 @@ def desktop() -> list[str]:
     return [str(f) for f in desktop.iterdir()]
 
 
+# Add a dynamic greeting resource
+@mcp.resource("greeting://{name}")
+def get_greeting(name: str) -> str:
+    """Get a personalized greeting"""
+    return f"Hello, {name}!"
+
+
 @mcp.tool()
 def add(a: int, b: int) -> int:
     """Add two numbers"""
