@@ -2,9 +2,10 @@
 
 from importlib.metadata import version
 
-
 from fastmcp.server.server import FastMCP
 from fastmcp.server.context import Context
+import fastmcp.server
+
 from fastmcp.client import Client
 from fastmcp.utilities.types import Image
 from . import client, settings
@@ -14,6 +15,7 @@ __all__ = [
     "FastMCP",
     "Context",
     "client",
+    "Client",
     "settings",
     "Image",
 ]
