@@ -1,12 +1,18 @@
 import json
 import logging
+import uuid
+from typing import Annotated, Any
 
+import pydantic_core
 import pytest
+from mcp.types import ImageContent, TextContent
 from pydantic import BaseModel
 
+from fastmcp import Context, FastMCP, Image
 from fastmcp.exceptions import NotFoundError, ToolError
 from fastmcp.tools import ToolManager
 from fastmcp.tools.tool import Tool
+from fastmcp.utilities.tests import temporary_settings
 
 
 class TestAddTools:
@@ -24,7 +30,6 @@ class TestAddTools:
         assert tool is not None
         assert tool.name == "add"
         assert tool.description == "Add two numbers."
-        assert tool.is_async is False
         assert tool.parameters["properties"]["a"]["type"] == "integer"
         assert tool.parameters["properties"]["b"]["type"] == "integer"
 
@@ -42,7 +47,6 @@ class TestAddTools:
         assert tool is not None
         assert tool.name == "fetch_data"
         assert tool.description == "Fetch data from URL."
-        assert tool.is_async is True
         assert tool.parameters["properties"]["url"]["type"] == "string"
 
     def test_pydantic_model_function(self):
@@ -63,14 +67,25 @@ class TestAddTools:
         assert tool is not None
         assert tool.name == "create_user"
         assert tool.description == "Create a new user."
-        assert tool.is_async is False
         assert "name" in tool.parameters["$defs"]["UserInput"]["properties"]
         assert "age" in tool.parameters["$defs"]["UserInput"]["properties"]
         assert "flag" in tool.parameters["properties"]
 
-    def test_add_invalid_tool(self):
+    async def test_tool_with_image_return(self):
+        def image_tool(data: bytes) -> Image:
+            return Image(data=data)
+
         manager = ToolManager()
-        with pytest.raises(AttributeError):
+        manager.add_tool_from_fn(image_tool)
+
+        tool = manager.get_tool("image_tool")
+        result = await tool.run({"data": "test.png"})
+        assert tool.parameters["properties"]["data"]["type"] == "string"
+        assert isinstance(result[0], ImageContent)
+
+    def test_add_noncallable_tool(self):
+        manager = ToolManager()
+        with pytest.raises(TypeError, match="not a callable object"):
             manager.add_tool_from_fn(1)  # type: ignore
 
     def test_add_lambda(self):
@@ -263,7 +278,6 @@ class TestCallTools:
         result = await manager.call_tool("double", {"n": 5})
         assert isinstance(result, list)
         assert len(result) == 1
-        from mcp.types import TextContent
 
         assert isinstance(result[0], TextContent)
         assert result[0].text == "10"
@@ -279,7 +293,6 @@ class TestCallTools:
         result = await manager.call_tool("add", {"a": 1})
         assert isinstance(result, list)
         assert len(result) == 1
-        from mcp.types import TextContent
 
         assert isinstance(result[0], TextContent)
         assert result[0].text == "2"
@@ -306,15 +319,6 @@ class TestCallTools:
 
         manager = ToolManager()
         manager.add_tool_from_fn(sum_vals)
-        # Try both with plain list and with JSON list
-        from mcp.types import TextContent
-
-        result = await manager.call_tool("sum_vals", {"vals": "[1, 2, 3]"})
-        assert isinstance(result, list)
-        assert len(result) == 1
-        assert isinstance(result[0], TextContent)
-        assert result[0].text == "6"
-        assert json.loads(result[0].text) == 6
 
         result = await manager.call_tool("sum_vals", {"vals": [1, 2, 3]})
         assert isinstance(result, list)
@@ -323,13 +327,30 @@ class TestCallTools:
         assert result[0].text == "6"
         assert json.loads(result[0].text) == 6
 
+    async def test_call_tool_with_list_int_input_legacy_behavior(self):
+        """Legacy behavior -- parse a stringified JSON object"""
+
+        def sum_vals(vals: list[int]) -> int:
+            return sum(vals)
+
+        manager = ToolManager()
+        manager.add_tool_from_fn(sum_vals)
+        # Try both with plain list and with JSON list
+
+        with temporary_settings(tool_attempt_parse_json_args=True):
+            result = await manager.call_tool("sum_vals", {"vals": "[1, 2, 3]"})
+            assert isinstance(result, list)
+            assert len(result) == 1
+            assert isinstance(result[0], TextContent)
+            assert result[0].text == "6"
+            assert json.loads(result[0].text) == 6
+
     async def test_call_tool_with_list_str_or_str_input(self):
         def concat_strs(vals: list[str] | str) -> str:
             return vals if isinstance(vals, str) else "".join(vals)
 
         manager = ToolManager()
         manager.add_tool_from_fn(concat_strs)
-        from mcp.types import TextContent
 
         # Try both with plain python object and with JSON list
         result = await manager.call_tool("concat_strs", {"vals": ["a", "b", "c"]})
@@ -338,29 +359,35 @@ class TestCallTools:
         assert isinstance(result[0], TextContent)
         assert result[0].text == "abc"
 
-        result = await manager.call_tool("concat_strs", {"vals": '["a", "b", "c"]'})
-        assert isinstance(result, list)
-        assert len(result) == 1
-        assert isinstance(result[0], TextContent)
-        assert result[0].text == "abc"
-
         result = await manager.call_tool("concat_strs", {"vals": "a"})
         assert isinstance(result, list)
         assert len(result) == 1
         assert isinstance(result[0], TextContent)
         assert result[0].text == "a"
 
-        result = await manager.call_tool("concat_strs", {"vals": '"a"'})
-        assert isinstance(result, list)
-        assert len(result) == 1
-        assert isinstance(result[0], TextContent)
-        assert result[0].text == '"a"'
+    async def test_call_tool_with_list_str_or_str_input_legacy_behavior(self):
+        """Legacy behavior -- parse a stringified JSON object"""
+
+        def concat_strs(vals: list[str] | str) -> str:
+            return vals if isinstance(vals, str) else "".join(vals)
+
+        manager = ToolManager()
+        manager.add_tool_from_fn(concat_strs)
+
+        with temporary_settings(tool_attempt_parse_json_args=True):
+            result = await manager.call_tool("concat_strs", {"vals": '["a", "b", "c"]'})
+            assert isinstance(result, list)
+            assert len(result) == 1
+            assert isinstance(result[0], TextContent)
+            assert result[0].text == "abc"
+
+            result = await manager.call_tool("concat_strs", {"vals": '"a"'})
+            assert isinstance(result, list)
+            assert len(result) == 1
+            assert isinstance(result[0], TextContent)
+            assert result[0].text == "a"
 
     async def test_call_tool_with_complex_model(self):
-        from mcp.types import TextContent
-
-        from fastmcp import Context
-
         class MyShrimpTank(BaseModel):
             class Shrimp(BaseModel):
                 name: str
@@ -368,37 +395,107 @@ class TestCallTools:
             shrimp: list[Shrimp]
             x: None
 
-        def name_shrimp(tank: MyShrimpTank, ctx: Context) -> list[str]:
+        def name_shrimp(tank: MyShrimpTank, ctx: Context | None) -> list[str]:
             return [x.name for x in tank.shrimp]
 
         manager = ToolManager()
         manager.add_tool_from_fn(name_shrimp)
 
-        result = await manager.call_tool(
-            "name_shrimp",
-            {"tank": {"x": None, "shrimp": [{"name": "rex"}, {"name": "gertrude"}]}},
-        )
-        assert isinstance(result, list)
-        assert len(result) == 1
-        assert isinstance(result[0], TextContent)
-        assert result[0].text == '["rex", "gertrude"]'
-        assert json.loads(result[0].text) == ["rex", "gertrude"]
+        mcp = FastMCP()
+        context = Context(fastmcp=mcp)
+
+        with context:
+            result = await manager.call_tool(
+                "name_shrimp",
+                {
+                    "tank": {
+                        "x": None,
+                        "shrimp": [{"name": "rex"}, {"name": "gertrude"}],
+                    }
+                },
+            )
 
-        result = await manager.call_tool(
-            "name_shrimp",
-            {"tank": '{"x": null, "shrimp": [{"name": "rex"}, {"name": "gertrude"}]}'},
-        )
         assert isinstance(result, list)
         assert len(result) == 1
         assert isinstance(result[0], TextContent)
-        assert result[0].text == '["rex", "gertrude"]'
-        assert json.loads(result[0].text) == ["rex", "gertrude"]
+        assert result[0].text == '[\n  "rex",\n  "gertrude"\n]'
+
+    async def test_call_tool_with_custom_serializer(self):
+        """Test that a custom serializer provided to FastMCP is used by tools."""
+
+        def custom_serializer(data: Any) -> str:
+            if isinstance(data, dict):
+                return f"CUSTOM:{json.dumps(data)}"
+            return json.dumps(data)
+
+        # Instantiate FastMCP with the custom serializer
+        mcp = FastMCP(tool_serializer=custom_serializer)
+        manager = mcp._tool_manager
+
+        def get_data() -> dict:
+            return {"key": "value", "number": 123}
+
+        manager.add_tool_from_fn(get_data)
+
+        result = await manager.call_tool("get_data", {})
+        assert isinstance(result, list)
+        assert len(result) == 1
+        assert isinstance(result[0], TextContent)
+        assert result[0].text == 'CUSTOM:{"key": "value", "number": 123}'
+
+    async def test_call_tool_with_list_result_custom_serializer(self):
+        """Test that a custom serializer provided to FastMCP is used by tools that return lists."""
+
+        def custom_serializer(data: Any) -> str:
+            if isinstance(data, list):
+                return f"CUSTOM:{json.dumps(data)}"
+            return json.dumps(data)
+
+        mcp = FastMCP(tool_serializer=custom_serializer)
+        manager = mcp._tool_manager
+
+        def get_data() -> list[dict]:
+            return [
+                {"key": "value", "number": 123},
+                {"key": "value2", "number": 456},
+            ]
+
+        manager.add_tool_from_fn(get_data)
+
+        result = await manager.call_tool("get_data", {})
+        assert isinstance(result, list)
+        assert len(result) == 1
+        assert isinstance(result[0], TextContent)
+        assert (
+            result[0].text
+            == 'CUSTOM:[{"key": "value", "number": 123}, {"key": "value2", "number": 456}]'
+        )
+
+    async def test_custom_serializer_fallback_on_error(self):
+        """Test that a broken custom serializer gracefully falls back."""
+
+        uuid_result = uuid.uuid4()
+
+        def custom_serializer(data: Any) -> str:
+            return json.dumps(data)
+
+        mcp = FastMCP(tool_serializer=custom_serializer)
+        manager = mcp._tool_manager
+
+        def get_data() -> uuid.UUID:
+            return uuid_result
+
+        manager.add_tool_from_fn(get_data)
+
+        result = await manager.call_tool("get_data", {})
+        assert isinstance(result, list)
+        assert len(result) == 1
+        assert isinstance(result[0], TextContent)
+        assert result[0].text == pydantic_core.to_json(uuid_result).decode()
 
 
 class TestToolSchema:
     async def test_context_arg_excluded_from_schema(self):
-        from fastmcp import Context
-
         def something(a: int, ctx: Context) -> int:
             return a
 
@@ -406,7 +503,24 @@ class TestToolSchema:
         tool = manager.add_tool_from_fn(something)
         assert "ctx" not in json.dumps(tool.parameters)
         assert "Context" not in json.dumps(tool.parameters)
-        assert "ctx" not in tool.fn_metadata.arg_model.model_fields
+
+    async def test_optional_context_arg_excluded_from_schema(self):
+        def something(a: int, ctx: Context | None) -> int:
+            return a
+
+        manager = ToolManager()
+        tool = manager.add_tool_from_fn(something)
+        assert "ctx" not in json.dumps(tool.parameters)
+        assert "Context" not in json.dumps(tool.parameters)
+
+    async def test_annotated_context_arg_excluded_from_schema(self):
+        def something(a: int, ctx: Annotated[Context | int | None, "ctx"]) -> int:
+            return a
+
+        manager = ToolManager()
+        tool = manager.add_tool_from_fn(something)
+        assert "ctx" not in json.dumps(tool.parameters)
+        assert "Context" not in json.dumps(tool.parameters)
 
 
 class TestContextHandling:
@@ -415,26 +529,20 @@ class TestContextHandling:
     def test_context_parameter_detection(self):
         """Test that context parameters are properly detected in
         Tool.from_function()."""
-        from fastmcp import Context
 
         def tool_with_context(x: int, ctx: Context) -> str:
             return str(x)
 
         manager = ToolManager()
-        tool = manager.add_tool_from_fn(tool_with_context)
-        assert tool.context_kwarg == "ctx"
+        manager.add_tool_from_fn(tool_with_context)
 
         def tool_without_context(x: int) -> str:
             return str(x)
 
-        tool = manager.add_tool_from_fn(tool_without_context)
-        assert tool.context_kwarg is None
+        manager.add_tool_from_fn(tool_without_context)
 
     async def test_context_injection(self):
         """Test that context is properly injected during tool execution."""
-        from mcp.types import TextContent
-
-        from fastmcp import Context, FastMCP
 
         def tool_with_context(x: int, ctx: Context) -> str:
             assert isinstance(ctx, Context)
@@ -444,18 +552,17 @@ class TestContextHandling:
         manager.add_tool_from_fn(tool_with_context)
 
         mcp = FastMCP()
-        ctx = mcp.get_context()
-        result = await manager.call_tool("tool_with_context", {"x": 42}, context=ctx)
-        assert isinstance(result, list)
-        assert len(result) == 1
-        assert isinstance(result[0], TextContent)
-        assert result[0].text == "42"
+        context = Context(fastmcp=mcp)
+
+        with context:
+            result = await manager.call_tool("tool_with_context", {"x": 42})
+            assert isinstance(result, list)
+            assert len(result) == 1
+            assert isinstance(result[0], TextContent)
+            assert result[0].text == "42"
 
     async def test_context_injection_async(self):
         """Test that context is properly injected in async tools."""
-        from mcp.types import TextContent
-
-        from fastmcp import Context, FastMCP
 
         async def async_tool(x: int, ctx: Context) -> str:
             assert isinstance(ctx, Context)
@@ -465,34 +572,65 @@ class TestContextHandling:
         manager.add_tool_from_fn(async_tool)
 
         mcp = FastMCP()
-        ctx = mcp.get_context()
-        result = await manager.call_tool("async_tool", {"x": 42}, context=ctx)
-        assert isinstance(result, list)
-        assert len(result) == 1
-        assert isinstance(result[0], TextContent)
-        assert result[0].text == "42"
+        context = Context(fastmcp=mcp)
+
+        with context:
+            result = await manager.call_tool("async_tool", {"x": 42})
+            assert isinstance(result, list)
+            assert len(result) == 1
+            assert isinstance(result[0], TextContent)
+            assert result[0].text == "42"
 
     async def test_context_optional(self):
         """Test that context is optional when calling tools."""
         from mcp.types import TextContent
 
-        from fastmcp import Context
-
-        def tool_with_context(x: int, ctx: Context | None = None) -> str:
-            return str(x)
+        def tool_with_context(x: int, ctx: Context | None) -> int:
+            return x
 
         manager = ToolManager()
         manager.add_tool_from_fn(tool_with_context)
         # Should not raise an error when context is not provided
-        result = await manager.call_tool("tool_with_context", {"x": 42})
-        assert isinstance(result, list)
-        assert len(result) == 1
-        assert isinstance(result[0], TextContent)
-        assert result[0].text == "42"
+
+        mcp = FastMCP()
+        context = Context(fastmcp=mcp)
+
+        with context:
+            result = await manager.call_tool("tool_with_context", {"x": 42})
+            assert isinstance(result, list)
+            assert len(result) == 1
+            assert isinstance(result[0], TextContent)
+            assert result[0].text == "42"
+
+    def test_parameterized_context_parameter_detection(self):
+        """Test that context parameters are properly detected in
+        Tool.from_function()."""
+
+        def tool_with_context(x: int, ctx: Context) -> str:
+            return str(x)
+
+        manager = ToolManager()
+        manager.add_tool_from_fn(tool_with_context)
+
+    def test_annotated_context_parameter_detection(self):
+        def tool_with_context(x: int, ctx: Annotated[Context, "ctx"]) -> str:
+            return str(x)
+
+        manager = ToolManager()
+        manager.add_tool_from_fn(tool_with_context)
+
+    def test_parameterized_union_context_parameter_detection(self):
+        """Test that context parameters are properly detected in
+        Tool.from_function()."""
+
+        def tool_with_context(x: int, ctx: Context | None) -> str:
+            return str(x)
+
+        manager = ToolManager()
+        manager.add_tool_from_fn(tool_with_context)
 
     async def test_context_error_handling(self):
         """Test error handling when context injection fails."""
-        from fastmcp import Context, FastMCP
 
         def tool_with_context(x: int, ctx: Context) -> str:
             raise ValueError("Test error")
@@ -501,9 +639,13 @@ class TestContextHandling:
         manager.add_tool_from_fn(tool_with_context)
 
         mcp = FastMCP()
-        ctx = mcp.get_context()
-        with pytest.raises(ToolError, match="Error executing tool tool_with_context"):
-            await manager.call_tool("tool_with_context", {"x": 42}, context=ctx)
+        context = Context(fastmcp=mcp)
+
+        with context:
+            with pytest.raises(
+                ToolError, match="Error executing tool tool_with_context"
+            ):
+                await manager.call_tool("tool_with_context", {"x": 42})
 
 
 class TestCustomToolNames:
